export @construct


function make_widget(binding)
    if binding.head != :(=)
        error("@construct syntax error.")
    end
    sym, expr = binding.args
    Expr(:(=), esc(sym),
         Expr(:call, widget, esc(expr), string(sym)))
end

function display_widgets(widgetvars)
    map(v -> Expr(:call, esc(:display), esc(v)), widgetvars)
end

function map_block(block, symbols)
    lambda = Expr(:(->), Expr(:tuple, symbols...),
                  block)
    :(map($lambda, $(map(s->:(obs($s)), symbols)...)))
end

function symbols(bindings)
    map(x->x.args[1], bindings)
end

"""
Modified version of the @manipulate macro from InteractNext.
Usage:

widgets, graphic = @construct for x in 1:10,
    y in slider(1:10)
    plot(x,y)
end

where widgets is a Widget_Container, the widget for x being accessible with widgets[:x], and graphic is the DOM containing Node of the plot.

Important: The graphic and all widgets must be included in the dom response for the observables to work. If we don't wish to display them, we can include them with Node(:div, widgets[:x], style=Dict(:display=>"none")) in the dom response.

"""
macro construct(expr)
    if expr.head != :for
        error("@construct syntax is @construct for ",
              " [<variable>=<domain>,]... <expression> end")
    end
    block = expr.args[2]
    if expr.args[1].head == :block
        bindings = expr.args[1].args
    else
        bindings = [expr.args[1]]
    end
    syms = symbols(bindings)

    widgets = map(make_widget, bindings)
    dict = Dict(zip(syms, 1:length(syms)))
    quote
        (Widget_Container([$(widgets...)], $(dict)), WebIO.render($(esc(map_block(block, syms)))))
    end
end

widget(x::Range, label="") = slider(x; label=label)
widget(x::Observable, label="") = x
widget(x::WebIO.Node{<:Any}, label="") = x
widget(x::WebIO.Scope, label="") = x #fix for WebIO update
widget(x::AbstractVector, label="") = togglebuttons(x, label=label)
widget(x::Associative, label="") = togglebuttons(x, label=label)
widget(x::Bool, label="") = checkbox(x, label=label)
widget(x::AbstractString, label="") = textbox(x, label=label, typ=AbstractString)
